import logging as log
import os
import sys
import traceback
from http.client import RemoteDisconnected
from time import sleep

import requests as r
import selenium.webdriver as webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
from selenium.webdriver.remote.webdriver import WebDriver
from urllib3.exceptions import MaxRetryError, ProtocolError
import requests

from feed.settings import nanny_params, browser_params
from feed.service import Client

logging = log.getLogger(__name__)


class BrowserService:
    driver: WebDriver
    retry_wait = 10
    retry_attempts = 10

    browser = Client("nanny", **nanny_params)

    def __init__(self, attempts=0, getContainerUrl="http://{host}:{port}/{api_prefix}/getContainer".format(**nanny_params)):
        """
        Request a port of the nanny service and then start a webdriver session
        :param attempts: will recursively try to get a container, do not populate
        """

        port = requests.get(getContainerUrl)
        try:
            if 100 < port.status_code < 400:
                logging.warning(f'nanny returned with {port.status_code}, response was {port.text}')

            else:
                self.port = port.text
                if not isinstance(port, int):
                    logging.info(f'Nanny container did not return a valid port number {port.text}')
                    raise Exception(port.text)
                # if no browser params specified then running in docker
                # => different host
                if browser_params['host'] is None:
                    url = f'http://worker-{self.port}:{browser_params.get["port"]}/wd/hub'
                else:
                    # otherwise always localhost
                    url = f'http://{browser_params["host"]}:{self.port}/wd/hub'

                options = Options()
                options.add_argument("--headless")
                self.startWebdriverSession(url, options, self.port)
            logging.info(f'Starting remote webdriver with {url}')
        except Exception as e:
            traceback.print_exc()
            if attempts < self.retry_attempts:
                sleep(self.retry_wait)
                self.__init__()
            else:
                logging.error(f'could connect to {getContainerUrl}')
                sys.exit()

    def startWebdriverSession(self, url, options, attempts=0):

        max_attempts = 10
        attempts += 1
        self.driver = webdriver.Remote(command_executor=url,
                                       desired_capabilities=DesiredCapabilities.CHROME,
                                       options=options)
        logging.info("started webdriver session")


def reportParameter(parameter_key=None):
    endpoint = "http://{host}:{port}/parametermanager/reportParameter/{}/{}/{}".format(
        os.getenv("NAME"),
        parameter_key,
        "leader",
        **nanny_params
    )
    r.get(endpoint)
